function F = hartley(xl, yl, xr, yr)
%  F = hartley(xl, yl, xr, yr)
%%%%%%%%%%%%%%%%%%%%%%% Metodo de Hartley %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Mtodo para determinao das linhas epipolares dado um par de imagens
% estereo.
% 
% Entradas da Funcao:
% xl - vetor com as coordenadas x dos pontos, na imagem da esquerda
% yl - vetor com as coordenadas y dos pontos, na imagem da esquerda
% xr - vetor com as coordenadas x dos pontos, na imagem da direita
% yr - vetor com as coordenadas y dos pontos, na imagem da direita
%
% Sadas da Funcao:
% F - Matriz Fundamental 
%




tam=length(xl);

x0l = mean(xl);
y0l = mean(yl);
x0r = mean(xr);
y0r = mean(yr);

a = sqrt(2)*tam/sum(sqrt((xl-x0l*ones(tam,1)).^2 + (yl-y0l*ones(tam,1)).^2));

Tl = eye(3);
Tl(1:2,3) = -a*[x0l;y0l];
% Tl = a*Tl;
% Tl(3,3) = 1;

a = sqrt(2)*tam/sum(sqrt((xr-x0r*ones(tam,1)).^2 + (yr-y0r*ones(tam,1)).^2));

Tr = eye(3);
Tr(1:2,3) = -a*[x0r;y0r];
% Tr = a*Tr;
% Tl(3,3) = 1;

xlt = [];
xrt = [];
ylt = [];
yrt = [];

for i=1:tam
    p = [xl(i);yl(i);1];
    pt = Tl*p;
    xlt = [xlt;pt(1)];
    ylt = [ylt;pt(2)];
    
    p = [xr(i);yr(i);1];
    pt = Tr*p;
    xrt = [xrt;pt(1)];
    yrt = [yrt;pt(2)];
end

P = [xlt.*xrt xlt.*yrt xlt ylt.*xrt ylt.*yrt ylt xrt yrt];

[V,D] = eig(P'*P);
[temp,I] = min(abs(diag(D)));

Ft = V(:,I);
Ft = [Ft;1];

Ft = [Ft(1:3)';Ft(4:6)';Ft(7:9)'];

Ft = Tl'*Ft*Tr;

clear V
[U,S,V] = svd(Ft);

[m,in] = min(abs(diag(S)));
 
S(in,in) = 0;
Fb = U*S*V';

F = Tl'*Fb*Tr;

