function F = luong_nonlin(xl, yl, xr, yr)
%  F = luong_nonlin(xl, yl, xr, yr)
%%%%%%%%%%%%%%%%%%% Metodo de Luong (no linear) %%%%%%%%%%%%%%%%%%%%%%%%%
% Mtodo para determinao das linhas epipolares dado um par de imagens
% estereo.
% 
% Entradas da Funcao:
% xl - vetor com as coordenadas x dos pontos, na imagem da esquerda
% yl - vetor com as coordenadas y dos pontos, na imagem da esquerda
% xr - vetor com as coordenadas x dos pontos, na imagem da direita
% yr - vetor com as coordenadas y dos pontos, na imagem da direita
%
% Sadas da Funcao:
% F - Matriz Fundamental 
%
% Raul Queiroz Feitosa - ltima reviso Maio/2011

F0 = oito_pontos(xl, yl, xr, yr);

pl=[xl(:)';yl(:)';ones(1,length(xl))];
pr=[xr(:)';yr(:)';ones(1,length(xl))];

tam=length(xl);

% - Calculo do epipolo l
[V,D] = eig(F0);
[temp,I] = min(diag(D));
el = V(:,I);

% - Calculo do epipolo r
[V,D] = eig(F0');
[temp,I] = min(diag(D));
er = V(:,I);

% - Calculo das coordenadas dos epipolos
alphal = el(1)/el(3);
betal = el(2)/el(3);

alphar = er(1)/er(3);
betar = er(2)/er(3);


% Estimativa de a, b, c e d
U=[       0             1             0            0;...
          0             0             0           -1;...
          0          -alphar          0          betar;...
          1             0             0            0;...
          0             0            -1            0;...
      -alphar           0           betar          0;...
       -betal        -alphal           0            0;...
          0             0           betal        alphal;...
     betal*alphar alphal*alphar -betal*betar -alphal*betar];
 V0=pinv(U)*(F0(:));

F=BuildFfromParameters(V0(1),V0(2),V0(3),V0(4),alphal,betal,alphar,betar);

options = optimset('Display','Off','MaxIter',500,'MaxFunEvals',500^2);

warning off
[V,residuo] = lsqnonlin(@(V) ...
    Distance2F(V,alphal,betal,alphar,betar,pl,pr),V0,[],[],options);
warning on

F=BuildFfromParameters(V(1),V(2),V(3),V(4),alphal,betal,alphar,betar);

[dl2,dr2]=Calcula_Distancia_Linha_Epipolar(pl,F,pr);


return



function D=Distance2F(V,alphal,betal,alphar,betar,pl,pr)

a = V(1);
b = V(2);
c = V(3);
d = V(4);

F=BuildFfromParameters(a,b,c,d,alphal,betal,alphar,betar);

[dl2,dr2]=Calcula_Distancia_Linha_Epipolar(pl,F,pr);

D=mean(sqrt([dl2 dr2]));

return


